# -------------------------------------------------------------------------------------------------
# This script is to compute the seasonal verification with CSDownscale for Focus-Africa WP4.
# 
# Forecasting system: ECMWF SEAS5
# Lead times: 0-5 months
# Observation: ERA5Land
# Variables: prlr, tas
# Frequency: monthly (aggregated to 3-month seasonal sum/mean)
# Regrid: Forecasts are interpolated to ERA5Land grid.
# Period: 1993-2016
# Calibration methods: RAW, bias, evmos and mse_min
# Skill metrics: corr, bss10, bss90, frpss
# Spatial domain: 0-60ºN, 40ºS-10ºN (the SADC region)
#
# Chung, July 2022
# -------------------------------------------------------------------------------------------------
rm(list = ls())

setwd('/esarchive/scratch/cchou/git/auto-s2s')
# Output plots directory
output.dir <- '/esarchive/scratch/cchou/FOCUS/seasonal-verification/output/auto-s2s/plots/'
# Replace with the path to the recipe you want to use
recipe_file <- "modules/Loading/testing_recipes/c_recipe1.yml"

source("modules/Loading/Loading.R")
source("modules/Calibration/Calibration.R")
source("modules/Skill/Skill.R")

# Load datasets
data <- load_datasets(recipe_file)

