# This function is for decadal-daily data loading.
# The time input (leadtimemin & leadtimemax) units in the recipe is month. For example,
# leadtimemin = 0 means the first month, and leadtimemax = 2 means the 3rd month.
# For daily data, if initial month is November, leadtimemin = 0 equals to c(1:30), 
# and leadtimemax = 2, which means January next year, equals to c(31+31:31+31+30).
get_daily_time_ind <- function(leadtimemin, leadtimemax, initial_month, sdates, calendar) {
  #TODO: calendar & leap year
  # E.g., time = 1:4 means Dec to March, then indices will be c(31:(31+30+31+28+31)) for non-leap yr and c(31:(31+30+31+28), (31+30+31+28+2):(31+30+31+28+2+30)) for leap yr. Calendar needs to be considered too.

  if (!calendar %in% c('360-day', '365_day', 'noleap', 'standard', 'proleptic_gregorian',
                       'gregorian')) 
    stop("The calendar is not recognized. Please contact maintainers.")

  total_months <- (initial_month + leadtimemin):(initial_month + leadtimemax)
  total_months <- ifelse(total_months > 12, total_months %% 12, total_months)
  total_months <- ifelse(total_months == 0, 12, total_months)

  if (calendar == '360-day') {
    daily_time_ind <- 1:(30 * length(total_months))

  } else {

    total_days_sum <- sum(days_in_month(total_months))
    if (leadtimemin == 0) {
      daily_time_ind <- 1:total_days_sum
    } else {
      before_start_months <- initial_month:(initial_month + leadtimemin - 1)
      before_start_months <- ifelse(before_start_months > 12, before_start_months %% 12, before_start_months)
      before_start_months <- ifelse(before_start_months == 0, 12, before_start_months)
      before_start_total_days_sum <- sum(days_in_month(before_start_months))
      daily_time_ind <- (before_start_total_days_sum + 1):(before_start_total_days_sum + total_days_sum)
    }


    if (calendar %in% c('365_day', 'noleap')) {
      # daily_time_ind
    } else if (calendar %in% c('standard', 'proleptic_gregorian', 'gregorian')) {

#      leap <- leap_year(sdates)
      sdates_full <- ymd(paste0(sdates, sprintf('%02d',initial_month), '01'))
      idx_min <- sdates_full + months(leadtimemin)
      idx_max <- sdates_full + months(leadtimemax + 1) - days(1)

      day0229 <- rep(NA, length(sdates))
      for (i_sdate in 1:length(sdates_full)) {
        days <- seq(idx_min[i_sdate],idx_max[i_sdate], by = 'days')
        day0229[i_sdate] <- sum(format(days, "%m%d") == "0229")
      }

      if (all(day0229 == 0)) {  # no 0229
        # daily_time_ind
      } else {
        # get extra indices and remove 0229 & extra indices after retrieval
        #TODO: Make Start() can handle time as a list or array for this case
        daily_time_ind <- c(daily_time_ind, 
                            (tail(daily_time_ind, 1) + 1):((tail(daily_time_ind, 1) + 1) + (max(day0229) - 1)))
      }
  
    }
  }

  return(daily_time_ind)
}


correct_daily_for_leap <- function(data = NULL, time_attr, return_time = TRUE) {
  # data & time_attr: [syear, time]
  #NOTE: return_time = TRUE, only return time_attr; FALSE, only return data

  new_tmp_time_attr <- array(NA, dim = dim(time_attr))
  if (!is.null(data)) {
    new_data <- array(NA, dim = dim(time_attr))
  } else {
    new_data <- NULL
  }
  for (i_sdate in 1:dim(time_attr)[1]) {
    ind_not_0229 <- !(format(time_attr[i_sdate, ], "%m%d") == "0229")
    new_tmp_time_attr[i_sdate, 1:sum(ind_not_0229)] <- time_attr[i_sdate, ][ind_not_0229]
    if (!is.null(data)) {
      new_data[i_sdate, 1:sum(ind_not_0229)] <- data[i_sdate, ][ind_not_0229]
    }
  }
  real_time_length <- min(which(is.na(new_tmp_time_attr),arr.ind = T)[, 2]) - 1
  new_tmp_time_attr <- new_tmp_time_attr[, 1:real_time_length]
  new_tmp_time_attr <- as.POSIXct(new_tmp_time_attr, tz = 'UTC', origin = '1970-01-01')
  if (!is.null(data)) {
    new_data <- new_data[, 1:real_time_length]
  }

  if (any(!new_tmp_time_attr %in% time_attr | any(is.na(new_tmp_time_attr)))) {
    stop("Wrong time attribute modification.")
  }
  if (return_time) {
    return(list(time_attr = new_tmp_time_attr))
  } else {
    return(list(data = new_data))
  }
}
