library(easyNCDF)

save_bias <- function(variable,
                      data,
                      fcst.sdate,
                      outfile,
                      leadtimes,
                      grid,
                      agg,
                      fcst.type) {

  lalo <- c('longitude', 'latitude') #decathlon subseasonal
		
  ## TODO: Sort out different aggregation cases
  # if (tolower(agg) == "global") {
  #   data <- Reorder(data, c(lalo,'time'))
  # } else {
  #   data <- Reorder(data, c('country', 'time'))
  # }
  
  if (variable %in% c("tas", "tasmin", "tasmax")) {
    obs <- data; units <- "ºC";
    var.longname <- "Temperature bias" 
  } else {
    data.conv <- convert_data(list(fcst=data,test=data),variable,leadtimes,fcst.type,"forecast")
    obs <- data.conv$data$fcst; units <- data.conv$units;
    var.longname <- data.conv$var.longname
    remove(data.conv)
  }

  if (tolower(agg) == "country"){
    dims <- c('Country', 'time')
    var.expname <- paste0(variable, '_country') 
    var.sdname <- paste0("Country-Aggregated ", var.longname)
  } else {
    dims <- c(lalo,'time')
    var.expname <- get_outname(variable,VARS_DICT)
    var.sdname <- var.longname
  }

  metadata <- list(obs = list(name = var.expname,
                              standard_name = var.sdname,
                              units = units))
  attr(obs, 'variables') <- metadata
  names(dim(obs)) <- dims 
  
  times <- get_times(fcst.type, leadtimes, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  if (tolower(agg) == "country") { 
    
    country <- get_countries(grid)
    ArrayToNc(list(country, time, time_step, obs), outfile)

  } else  {

    latlon <- get_latlon(grid$lat, grid$lon)
    ArrayToNc(list(latlon$lat, latlon$lon, time, obs, time_step), outfile)

  }
}

save_obs_country_file <-
  function(variable,
           obs,
           fcst.sdate,
           outfile,
           fcst.type,
           monthnames) {

    if (fcst.type == 'seasonal'){
      mask.path <- 'masks/mask_europe_system5_2.Rdata'
    } else {
      mask.path <- 'masks/mask_europe_S2S_ecmwf.Rdata'
    }
    
    load(mask.path)
    
    ifelse(exists("lonlat_dctln"),
	lalo <- c('longitude','latitude'), #decathlon subseasonal
	lalo <- c('latitude','longitude')) #no decathlon

    obs <- Reorder(obs, c(lalo,'time'))
#    obs <- Reorder(obs, c('latitude','longitude','time'))

    obs.country <-Apply(data=list(pointdata=obs),
#                                  target_dims=c('latitude','longitude'),
				  target_dims=lalo,
                                  output_dims=c('country'),
                                  mask.path=mask.path, 
                                  ncores=2,
                                  split_factor=1,
                                  fun = Country_mean)[[1]]

  vars <- yaml.load_file(VARS_DICT)$vars
  units <- vars[[variable]]$units
  var.longname <- vars[[variable]]$longname

  metadata <- list(obs.country = 
      list(
          name = paste0(variable, "_country"),
          standard_name = paste0(var.longname, " (Country-Aggregated)"),
          units = units
      )
  )
    
  attr(obs.country, 'variables') <- metadata
  names(dim(obs.country)) <- c('Country', 'time')
  
  times <- get_times(fcst.type, monthnames, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  country <- 1:length(europe.countries.iso)
  dim(country) <- length(country)
  metadata <- list( country = list(
                    standard_name = paste(europe.countries.iso, collapse=" "),
		            units = 'Country ISO 3166-1 alpha 3 Code')) #if region, these units are incorrect.
  attr(country, 'variables') <- metadata
  names(dim(country)) <- 'Country'


  ArrayToNc(list(country,time,time_step,obs.country),
            outfile)
 
 }

save_obs <- function(variable,
                     data,
                     fcst.sdate,
                     outfile,
                     leadtimes,
                     grid,
                     agg,
                     fcst.type) {

  lalo <- c('longitude','latitude') #decathlon subseasonal

  if (tolower(agg) == "global"){
    data <- Reorder(data, c(lalo,'time'))
  } else {
    data <- Reorder(data, c('country', 'time'))
  }

  data.conv <- convert_data(list(fcst=data,test=data),variable,leadtimes,fcst.type,"forecast")
  obs <- data.conv$data$fcst; units <- data.conv$units;
  var.longname <- data.conv$var.longname
  remove(data.conv)

  if (tolower(agg) == "country"){
    dims <- c('Country', 'time')
    var.expname <- paste0(variable, '_country') 
    var.sdname <- paste0("Country-Aggregated ", var.longname)
  } else {
    dims <- c(lalo,'time')
    var.expname <- get_outname(variable,VARS_DICT)
    var.sdname <- var.longname
  }

  metadata <- list(obs = list(name = var.expname,
                              standard_name = var.sdname,
                              units = units))
  attr(obs, 'variables') <- metadata
  names(dim(obs)) <- dims 
  
  times <- get_times(fcst.type, leadtimes, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  if (tolower(agg) == "country") { 
    
    country <- get_countries(grid)
    ArrayToNc(list(country, time, time_step, obs), outfile)

  } else  {

    latlon <- get_latlon(grid$lat, grid$lon)
    ArrayToNc(list(latlon$lat, latlon$lon, time, obs, time_step), outfile)

  }
}


save_forecast <- function(variable,
                          fcst,
                          fcst.sdate,
                          outfile,
                          leadtimes,
                          grid,
                          agg,
                          fcst.type) {

  lalo <- c('longitude','latitude') #decathlon subseasonal

  if (tolower(agg) == "global"){
    fcst <- Reorder(fcst, c(lalo,'member',
                          'time'))
  } else {
    fcst <- Reorder(fcst, c('country','member', 'time'))
  }

  fcst.conv <- convert_data(list(fcst=fcst,test=fcst),variable,leadtimes,fcst.type,"forecast")
  fcst <- fcst.conv$data$fcst; units <- fcst.conv$units;
  var.longname <- fcst.conv$var.longname
  remove(fcst.conv)

  if (tolower(agg) == "country"){
    dims <- c('Country', 'ensemble', 'time')
    var.expname <- paste0(variable, '_country') 
    var.sdname <- paste0("Country-Aggregated ", var.longname)
  } else {
    dims <- c(lalo,'ensemble', 'time')
    var.expname <- get_outname(variable,VARS_DICT)
    var.sdname <- var.longname
  }

  metadata <- list(fcst = list(name = var.expname,
                               standard_name = var.sdname,
                               units = units))
  attr(fcst, 'variables') <- metadata
  names(dim(fcst)) <- dims 
  
  times <- get_times(fcst.type, leadtimes, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  if (tolower(agg) == "country") { 
    
    country <- get_countries(grid)
    ArrayToNc(list(country, time, time_step, fcst), outfile)

  } else  {

    latlon <- get_latlon(grid$lat, grid$lon)
    ArrayToNc(list(latlon$lat, latlon$lon, time, fcst, time_step), outfile)

  }
}

save_probs <- function(variable,
                       probs,
                       fcst.sdate,
                       outfile,
                       monthnames,
                       grid,
                       agg,
                       fcst.type) {
  
  lalo <- c('longitude','latitude') #decathlon subseasonal

  if (tolower(agg) == "global"){
    probs <-  lapply(probs, function(x){
                          Reorder(x, c('bin',lalo, 'time'))})
  }

  pbn <- Subset(probs$tercile, 'bin', list(1), drop='selected')
  pn <-  Subset(probs$tercile, 'bin', list(2), drop='selected')
  pan <- Subset(probs$tercile, 'bin', list(3), drop='selected')
  p10 <- Subset(probs$extreme, 'bin', list(1), drop='selected')
  p90 <- Subset(probs$extreme, 'bin', list(3), drop='selected')
   
  pn.sdname <- paste('Probability below normal category ', sep='');
  pan.sdname <- paste('Probability above normal category ', sep='');
  pbn.sdname <- paste('Probability normal category ', sep='');
  p10.sdname <- paste('Probability below extreme category ', sep='');
  p90.sdname <- paste('Probability above extreme category  ', sep='');
  
  if (tolower(agg) == "country"){
    dims <- c('Country', 'time')
    pn.sdanme <- paste0('Country-Aggregated ', pn.sdname) 
    pbn.sdanme <- paste0('Country-Aggregated ', pbn.sdname) 
    pan.sdanme <- paste0('Country-Aggregated ', pan.sdname) 
    p10.sdanme <- paste0('Country-Aggregated ', p10.sdname) 
    p90.sdanme <- paste0('Country-Aggregated ', p90.sdname) 
  } else {
    dims <- c(lalo, 'time')
    pn.sdanme <-  paste0('Global ', pn.sdname) 
    pbn.sdanme <- paste0('Global ', pbn.sdname) 
    pan.sdanme <- paste0('Global ', pan.sdname) 
    p10.sdanme <- paste0('Global ', p10.sdname) 
    p90.sdanme <- paste0('Global ', p90.sdname) 
  }

  metadata <- list(pbn = list(name = 'prob_bn',
                                        standard_name = pbn.sdname ),
                   pn = list(name = 'prob_n',
                                        standard_name = pn.sdname),
                   pan = list(name = 'prob_an',
                                        standard_name = pan.sdname),
                   p10 = list(name = 'prob_bp10',
                                        standard_name = p10.sdname),
                   p90 = list(name = 'prob_ap90',
                                        standard_name = p90.sdname))

  attr(pbn, 'variables') <- metadata[1]
  attr(pn, 'variables') <- metadata[2]
  attr(pan, 'variables') <- metadata[3]
  attr(p10, 'variables') <- metadata[4]
  attr(p90, 'variables') <- metadata[5]

  names(dim(pbn)) <- dims 
  names(dim(pn))  <- dims 
  names(dim(pan)) <- dims 
  names(dim(p10)) <- dims 
  names(dim(p90)) <- dims 
        
  times <- get_times(fcst.type, monthnames, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  if (tolower(agg) == "country") { 
    
    country <- get_countries(grid)
    ArrayToNc(list(country, time, pbn, pn, pan, p10, p90, time_step), outfile)

  } else  {

    latlon <- get_latlon(grid$lat, grid$lon)
    latitude <- latlon$lat; longitude <- latlon$lon
    ArrayToNc(list(latitude, longitude, time, pbn, pn, pan, p10, p90,
    time_step), outfile)
  }
 
 }

get_times <- function (fcst.type, leadtimes, sdate){

  switch(tolower(fcst.type),
         "seasonal"     = {len <- length(leadtimes); ref <- 'months since '; 
                           stdname <- paste(strtoi(leadtimes), collapse=", ")},
         "sub_obs"     = {len <- 52; ref <- 'week of the year '; 
                           stdname <- paste(strtoi(leadtimes), collapse=", ")},
         "subseasonal"     = {len <- 4; ref <- 'weeks since ';
                           stdname <- ''}
  ) 

  time <- 1:len
  dim(time) <- length(time)
  #metadata <- list(time = list(standard_name = stdname,
  metadata <- list(time = list(
                               units = paste0(ref, sdate, ' 00:00:00')))
  attr(time, 'variables') <- metadata
  names(dim(time)) <- 'time'
  
  time_step <- 1
  dim(time_step) <- length(time_step)
  metadata <- list(time_step = list(units = paste0(
                                    ref, sdate, ' 00:00:00')))
  attr(time_step, 'variables') <- metadata
  names(dim(time_step)) <- 'time_step'

  sdate <- 1:length(sdate)
  dim(sdate) <- length(sdate)
  metadata <- list(sdate = list(standard_name = paste(strtoi(sdate), collapse=", "),
                               units = paste0('Init date')))
  attr(sdate, 'variables') <- metadata
  names(dim(sdate)) <- 'sdate'

  return(list(time_step=time_step, time=time, sdate=sdate))

}

get_countries <- function(europe.countries.iso){

    country <- 1:length(europe.countries.iso)
    dim(country) <- length(country)
    metadata <- list( country = list(
                                standard_name = paste(europe.countries.iso, collapse=" "),
    	                        units = 'Country ISO 3166-1 alpha 3 Code'))
    attr(country, 'variables') <- metadata
    names(dim(country)) <- 'Country'
    return(country)

}

get_latlon <- function(lat, lon){

    longitude <- lon
    dim(longitude) <- length(longitude)
    metadata <- list(longitude = list(units = 'degrees_east'))
    attr(longitude, 'variables') <- metadata
    names(dim(longitude)) <- 'longitude'
    
    latitude <- lat
    dim(latitude) <- length(latitude)
    metadata <- list(latitude = list(units = 'degrees_north'))
    attr(latitude, 'variables') <- metadata
    names(dim(latitude)) <- 'latitude'

    return(list(lat=latitude,lon=longitude))

}

save_metrics <- function(variable,
                         skill,
                         fcst.sdate,
                         grid,
                         outfile,
                         monthnames,
                         fcst.type,
                         agg) 
{
  
  lalo <- c('longitude', 'latitude')

  ## TODO: Sort out aggregation
  if (tolower(agg) == "global") {
    skill <-  lapply(skill, function(x){
                     Reorder(x[[1]], c(lalo, 'time'))})
  }

  for (i in 1:length(skill)) {

    metric <- names(skill[i])
    if (tolower(agg) == "country"){
      sdname <- paste0(names(metric), " region-aggregated metric")
      dims <- c('Country', 'time')
    } else {
      sdname <- paste0(names(metric), " grid point metric")
      dims <- c(lalo, 'time')
    }
    metadata <- list(name = metric, standard_name = sdname)

    attr(skill[i], 'variables') <- metadata        
    names(dim(skill[[i]])) <- dims 
  }

  times <- get_times(fcst.type, monthnames, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  if (tolower(agg) == "country") { 
    
    country <- get_countries(grid)
    ArrayToNc(append(country, time, skill, time_step), outfile)

  } else  {

    latlon <- get_latlon(grid$lat, grid$lon)
    vars <- list(latlon$lat, latlon$lon, time)
    vars <- c(vars, skill, list(time_step))
    ArrayToNc(vars, outfile)
  }
}

convert_seasosnal_prlr <- function(data2convert, leadtimes,filetype){

    ind <- 1:length(leadtimes)
    #dates <- paste0(leadtimes,"01")
    # computes the last day of the month
    lastday <- sapply(ind, function(x) 
                      {as.integer(substr((seq(as.Date(leadtimes[x],"%Y%m%d"), 
                                               length=2,by="months")-1)[2], 
                                        9, 10))})

    if (filetype == "terciles"){
      ter <- sapply(ind, function(x) {
                Subset(data2convert$tercile, along='time',
                indices=x,drop='selected')*1000*3600*24*lastday[x]},
                simplify='array')
      ext <- sapply(ind, function(x) {
                Subset(data2convert$extreme, along='time',
                indices=x,drop='selected')*1000*3600*24*lastday[x]},
                simplify='array') 

      data2convert <- list(tercile=ter, extreme=ext)
    } else {
    
      ens <- sapply(ind, function(x) {
                Subset(data2convert$fcst, along='time',
                indices=x,drop='selected')*1000*3600*24*lastday[x]},
                simplify='array')

      data2convert <- list(fcst=ens)
    }

  return(data2convert)
}

convert_data <- function(data,variable, leadtimes, fcst.type,filetype){

    vars <- yaml.load_file(VARS_DICT)$vars
    units <- vars[[variable]]$units
    var.longname <- vars[[variable]]$longname

    if (variable %in% c("tas","tasmin","tasmax")){
      data <- lapply(data, function(x){ x - 273.15})
    } else if (variable %in% c("psl")){
      data <- lapply(data, function(x){ x/100})
    } else {
      print("WARNING: NO DATA CONVERSION APPLIED")
    }


    return(list(data=data, units=units, var.longname=var.longname))

} 


## TODO: implement lists as in save_metrics
save_terciles <- function(variable,
                          terciles,
                          fcst.sdate,
                          grid,
                          outfile,
                          leadtimes,
                          fcst.type,
                          agg) {

  lalo <- c('longitude','latitude') #decathlon subseasonal

  if (tolower(agg) == "global"){
    terciles <-  lapply(terciles, function(x){
                          Reorder(x, c('bin',lalo, 'time'))})
  }

  terciles.conv <- convert_data(terciles,variable,leadtimes,fcst.type,"terciles")
  terciles <- terciles.conv$data; units <- terciles.conv$units;
  var.longname <- terciles.conv$var.longname
  remove(terciles.conv)
  
  p33 <- Subset(terciles$tercile, 'bin', list(1), drop='selected')

  p66 <- Subset(terciles$tercile, 'bin', list(2), drop='selected')
  p10 <- Subset(terciles$extreme, 'bin', list(1), drop='selected')
  p90 <- Subset(terciles$extreme, 'bin', list(2), drop='selected')

  p33.sdname <- paste('Lower Tercile ', sep='');
  p66.sdname <- paste('Upper Tercile ', sep='');
  p10.sdname <- paste('Lower extreme', sep='');
  p90.sdname <- paste('Upper extreme', sep='');
  
  if (tolower(agg) == "country"){
    dims <- c('Country', 'time')
    p33.sdanme <- paste0('Country-Aggregated ', p33.sdname) 
    p66.sdanme <- paste0('Country-Aggregated ', p66.sdname) 
    p10.sdanme <- paste0('Country-Aggregated ', p10.sdname) 
    p90.sdanme <- paste0('Country-Aggregated ', p90.sdname) 
  } else {
    dims <- c(lalo, 'time')
    p33.sdanme <- paste0('Global ', p33.sdname) 
    p66.sdanme <- paste0('Global ', p66.sdname) 
    p10.sdanme <- paste0('Gloabl ', p10.sdname) 
    p90.sdanme <- paste0('Global ', p90.sdname) 
  }

  metadata <- list(pbn = list(name = 'p33',
                                        standard_name = p33.sdname ),
                   pn = list(name = 'p66',
                                        standard_name = p66.sdname),
                   pan = list(name = 'p10',
                                        standard_name = p10.sdname),
                   p10 = list(name = 'p90',
                                        standard_name = p90.sdname))

  attr(p33, 'variables') <- metadata[1]
  attr(p66, 'variables') <- metadata[2]
  attr(p10, 'variables') <- metadata[3]
  attr(p90, 'variables') <- metadata[4]

  names(dim(p33)) <- dims
  names(dim(p66)) <- dims
  names(dim(p10)) <- dims
  names(dim(p90)) <- dims

  times <- get_times(fcst.type, leadtimes, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  if (tolower(agg) == "country") { 
    
    country <- get_countries(grid)
    ArrayToNc(list(country, time, p33, p66, p10, p90, time_step), outfile)

  } else  {

    latlon <- get_latlon(grid$lat, grid$lon)
    ArrayToNc(list(latlon$lat, latlon$lon, time, p33, p66, p10, p90, time_step), outfile)
  }
}
