# This module should calculate verification metrics at any time of the workflow
# It should implement different verification metrics:
# - FRPSS and RPSS
# - FCRPSS and CRPSS
# - enscorr
# - bias
# - reliability diagram
# - ask Carlos which decadal metrics he is currently using

source("modules/Skill/s2s.metrics.R")

## TODO: Implement this in the future
## Which parameter are required?
# if (!("obs" %in% ls()) || is.null(obs)) {
#   error(logger,
#         "There is no object 'obs' in the global environment or it is NULL")
# }
# if (stream == "fcst" && (!("fcst" %in% ls()) || is.null(fcst))) {
#   error(logger,
#         "There is no object 'fcst' in the global environment or it is NULL")
# }
# if (!("hcst" %in% ls()) || is.null(hcst)) {
#   error(logger,
#         "There is no object 'hcst' in the global environment or it is NULL")
# }
# if (!("metric" %in% ls()) || is.null(metric)) {
#   warn(logger,
#        "Verification metric not found and it is set as 'EnsCorr'.")
#   metric <- 'EnsCorr'
# }
# if (metric %in% c('FRPSS', 'RPSS')) {
#   metric_fun <- "veriApply"
#   metric_method <- "FairRpss"
# } else if (metric %in% c("FCRPSS", "CRPSS")) {
#   metric_fun <- "veriApply"
# } else if (metric %in% c("EnsCorr", "EnsCor")) {
#   metric_fun <- "veriApply"
#   metric_method <- "EnsCorr"
# #...
# } else {
#   error(logger, "Unknown verification metric defined in the recipe.")
#   metric_fun <- 'NotFound'
# }
# info(logger, paste("#-------------------------- ", "\n",
#                    " running Skill module ", "\n",
#                    " it can call ", metric_fun ))

compute_skill_metrics <- function(exp, obs, recipe, na.rm = T, ncores = 1) {
  # exp: s2dv_cube containing the hindcast
  # obs: s2dv_cube containing the observations
  # recipe: auto-s2s recipe as provided by read_yaml
  
  ## TODO: Adapt time_dims to subseasonal case
  time_dim <- 'syear'
  memb_dim <- 'ensemble'
  metrics <- tolower(recipe$Analysis$Workflow$Skill$metric)
  skill_metrics <- list()
  for (metric in strsplit(metrics, " ")[[1]]) {
    # Whether the fair version of the metric is to be computed
    if (metric %in% c('frps', 'frpss', 'bss10', 'bss90')) {
      Fair <- T
    } else {
      Fair <- F
    }
    # Ranked Probability Score and Fair version
    if (metric %in% c('rps', 'frps')) {
      skill <- RPS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
          	 Fair = Fair, ncores = ncores)
      skill_metrics[[ metric ]] <- list(skill)

    # Ranked Probability Skill Score and Fair version
    } else if (metric %in% c('rpss', 'frpss')) {
      skill <- RPSS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
          	  Fair = Fair, ncores = ncores)
      skill_metrics[[ metric ]] <- list(skill$rpss)
      skill_metrics[[ paste0(metric, "_significance") ]] <- list(skill$sign)

    # Brier Skill Score - 10th percentile
    } else if (metric == 'bss10') {
      skill <- RPSS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
          	  prob_thresholds = 0.1, Fair = Fair, ncores = ncores)
      skill_metrics[[ metric ]] <- list(skill$rpss)
      skill_metrics[[ paste0(metric, "_significance") ]] <- list(skill$sign)

    # Brier Skill Score - 90th percentile
    } else if (metric == 'bss90') {
      skill <- RPSS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
          	  prob_thresholds = 0.9, Fair = Fair, ncores = ncores)
      skill_metrics[[ metric ]] <- list(skill$rpss)
      skill_metrics[[ paste0(metric, "_significance") ]] <- list(skill$sign)

    # Ensemble mean correlation
    } else if (metric == 'enscorr') {
      ## TODO: Implement option for Kendall and Spearman methods?
      skill <- s2dv::Corr(exp$data, obs$data, 
          		  dat_dim = 'dat', 
          		  time_dim = time_dim, 
          		  method = 'pearson', 
          		  memb_dim = memb_dim,
          		  ncores = ncores)
      skill_metrics[[ metric ]] <- list(skill$corr)
      skill_metrics[[ paste0(metric, "_p.value") ]] <- list(skill$p.val)
      skill_metrics[[ paste0(metric, "_conf.low") ]] <- list(skill$conf.lower)
      skill_metrics[[ paste0(metric, "_conf.up") ]] <- list(skill$conf.upper)
    # SpecsVerification metrics  
    } else if (grepl("specs", metric, fixed = TRUE)) {
      # Compute SpecsVerification version of the metrics
      ## Retain _specs in metric name for clarity?
      metric <- (strsplit(metric, "_"))[[1]][1] # Get metric name
      skill <- Compute_verif_metrics(exp$data, obs$data,
          			   skill_metrics = metric,
          			   verif.dims=c("syear", "sday", "sweek"),
          			   na.rm = na.rm,
          			   ncores = ncores)
      skill_metrics[[ metric ]] <- skill

    }

  }

  print("##### SKILL METRIC COMPUTATION COMPLETE #####")

  return(skill_metrics)

}
